(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(*  This example is based on an example designed for OCamlP3l.         *)
(*                                                                     *)
(***********************************************************************)

(* $Id: config.ml,v 1.12 2014/01/21 09:21:18 fclement Exp $ *)

type graphical_output = Window | Postscript | No;;

let graphical_output_of_string = function
  | "window" -> Window
  | "ps" | "postscript" -> Postscript
  | "no" | "none" -> No
  | s -> failwith (Printf.sprintf "Config: Unknown graphical output %s" s)
and string_of_graphical_output = function
  | Window -> "window"
  | Postscript -> "postscript"
  | No -> "none"
;;

type configuration = {
    rdec_config_filename : string;
    result_basename : string;
    result_dir : string;
    worker_dir : string;
    worker_exe : string;
    graphical_output : graphical_output;
    outer_itermax : int;
    inner_itermax : int;
    display_step : int;
    a_while : int;
  }
;;

let configuration_of s1 s2 s3 s4 s5 fname i1 i2 i3 i4 =
  let ib =
    try Scanf.Scanning.from_file fname
    with Sys_error s -> prerr_endline s; exit 6 in
  let go = Scanf.bscanf ib "%s" graphical_output_of_string in
  {rdec_config_filename = s1;
   result_basename = s2;
   result_dir = s3;
   worker_dir = s4;
   worker_exe = s5;
   graphical_output = go;
   outer_itermax = i1;
   inner_itermax = i2;
   display_step = i3;
   a_while = i4}
;;

let parse_configuration_file fname =
  let ib =
    try Scanf.Scanning.from_file fname
    with Sys_error s -> prerr_endline s; exit 6 in
  Scanf.bscanf ib
    "rdec_config_filename = %S; \
     result_basename = %S; \
     result_dir = %S; \
     worker_dir = %S; \
     worker_exe = %S; \
     graphical_output = %S; \
     outer_itermax = %d; \
     inner_itermax = %d; \
     display_step = %d; \
     a_while = %d; "
    configuration_of
;;

let print_configuration_file ob cfg =
  Printf.bprintf ob "\
     rdec_config_filename = %S;\n\
     result_basename = %S;\n\
     result_dir = %S;\n\
     worker_dir = %S;\n\
     worker_exe = %S;\n\
     graphical_output = %S;\n\
     outer_itermax = %d;\n\
     inner_itermax = %d;\n\
     display_step = %d;\n\
     a_while = %d;\n"
    cfg.rdec_config_filename
    cfg.result_basename
    cfg.result_dir
    cfg.worker_dir
    cfg.worker_exe
    (string_of_graphical_output cfg.graphical_output)
    cfg.outer_itermax
    cfg.inner_itermax
    cfg.display_step
    cfg.a_while
;;

let string_of_configuration_file cfg =
  let ob = Buffer.create 1024 in
  print_configuration_file ob cfg;
  Buffer.contents ob
;;
