(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl.ml,v 1.34 2014/01/21 09:21:15 fclement Exp $ *)

type ('a, 'b) sum = ('a, 'b) Sk.sum = Inl of 'a | Inr of 'b;;

type ('a, 'b) skel = ('a, 'b) Sk.Skl.t;;
type ('a, 'b) t = ('a, 'b) skel;;

(* Skeleton defaults *)

let default_color = 1;;

(* Compiler's primitives *)

let atom (params, llexp) =
  Sk.c_atom params llexp default_color
;;

(* Some internal functions *)

let to_sarray_gen = fun () -> Sklstream.to_sarray;;
let of_sarray_gen = fun () -> Sklstream.of_sarray;;

Expor.register to_sarray_gen;;
Expor.register of_sarray_gen;;

let get_exported f =
  match Expor.is_registered f with
  | None -> assert false
  | Some x -> atom ((), x)
;;

(* Definition of the skeletons *)

(* Builtin identity *)
let id = Sk.c_id;;

(* Implemented skeletons *)
let pipeline skl1 skl2 =
  Sk.c_pipe skl1 skl2
;;

let ( ||| ) a b = pipeline a b;;

let product skl1 skl2 =
  Sk.c_prod skl1 skl2
;;

let ( *** ) a b = product a b;;

let sum (skl1, skl2) =
  Sk.c_sum skl1 skl2
;;

let ( +++ ) a b = sum (a, b);;

let farm_i (skl_gen, n) =
  if n <= 0 then failwith "farm: non positive number of workers." else
  let rec loop n p =
    if n = 1 then skl_gen p else
    let r = n mod 2
    and q = n / 2 in
    Sk.c_par (loop q p) (loop (q + r) (q + p)) in
  loop n 0
;;

let farm (skl, n) = farm_i ((fun _ -> skl), n);;

let farm_vector (skl, n) =
  let stream_of_vstream = get_exported of_sarray_gen in
  let vstream_of_stream = get_exported to_sarray_gen in
  stream_of_vstream ||| farm (skl, n) ||| vstream_of_stream
;;

let rails skla =
  let stream_of_vstream = get_exported of_sarray_gen in
  let vstream_of_stream = get_exported to_sarray_gen in
  stream_of_vstream ||| Sk.c_rails skla ||| vstream_of_stream
;;

(* (* Unused. *)
let reduce_vector _ = failwith "not yet implemented";;
*)

let loop (p, skl) = Sk.c_loop p skl;;

(* Color handling *)

let colorize skl color =
  match skl with
  | Sk.Skl.Par _ | Sk.Skl.Rails _ | Sk.Skl.Pipe _
  | Sk.Skl.Id _ | Sk.Skl.Prod _ | Sk.Skl.Loop _
  | Sk.Skl.Sum _ ->
    (* We cannot act on them *)
    skl
  | Sk.Skl.Atom atom ->
    Sk.colorize atom color
;;
