(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: skl.mli,v 1.27 2014/01/21 09:21:15 fclement Exp $ *)

(* New playground, fresher *)

type ('a, 'b) sum = ('a, 'b) Sk.sum = Inl of 'a | Inr of 'b;;

type ('a, 'b) skel = ('a, 'b) Sk.Skl.t;;
type ('a, 'b) t = ('a, 'b) skel;;


(* Creating skeletons *)

(* identity is builtin, it is optimized *)
val id : unit -> ('a, 'a) skel;;

(* INTERNAL *)
  val atom : ('x * ('x, 'a, 'b) Expor.llexpr) -> ('a, 'b) skel;;

(* Composing skeletons *)

val farm : ('a, 'b) skel * int -> ('a, 'b) skel;;
val farm_i : ((int -> ('a, 'b) skel) * int) -> ('a, 'b) skel;;
val farm_vector : ('a, 'b) skel * int -> ('a array, 'b array) skel;;
val loop : ('a, bool) skel * ('a, 'a) skel -> ('a, 'a) skel;;
val pipeline : ('a, 'b) skel -> ('b, 'c) skel -> ('a, 'c) skel;;
val ( ||| ) : ('a, 'b) skel -> ('b, 'c) skel -> ('a, 'c) skel;;
val product :
    ('a0, 'b0) skel -> ('a1, 'b1) skel -> ('a0 * 'a1, 'b0 * 'b1) skel;;
val ( *** ) :
    ('a0, 'b0) skel -> ('a1, 'b1) skel -> ('a0 * 'a1, 'b0 * 'b1) skel;;
val rails : (('a, 'b) skel) array -> ('a array, 'b array) skel;;
val sum : (('a, 'c) skel * ('b, 'c) skel) -> (('a, 'b) sum, 'c) skel;;
val ( +++ ) : ('a, 'c) skel -> ('b, 'c) skel -> (('a, 'b) sum, 'c) skel;;

(* Coloring atom skeletons *)

val colorize : ('a, 'b) skel -> int -> ('a, 'b) skel;;
