(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: spawnlib.ml,v 1.25 2014/01/21 09:21:16 fclement Exp $ *)

(* Tools *)

let noimp () = failwith "Not implemented";;

let fail msg =
  Log.say Log.Err "%s" msg;
  exit 2
;;

let maybe x err f =
  match x with
  | Some x -> f x
  | None -> err ()
;;

(* Process management *)

let install_sighandlers _x = noimp ();;

let spawn_sock f x =
  let sock, addr = IO.create_bound_socket None in
  Unix.listen sock 1;
  match IO.fork () with
  | IO.Child ->
    f (fst (IO.wait_connection sock true)) x;
    exit 0
  | IO.Parent _ ->
    Unix.close sock;
    IO.get_port addr
;;

(* let spawn_pipe f x = *)
(*   let ic1, oc1 = cpipe () in *)
(*   let ic2, oc2 = cpipe () in *)
(*   match fork () with *)
(*   | Child -> *)
(*     (try f (ic1, oc2) x with _ -> ()); *)
(*     exit 0 *)
(*   | Parent _ -> *)
(*     (ic2, oc1) *)
(* ;; *)

(* Master-slave communications *)

type ('state, 'req) request_handler = 'req -> 'state;;
type ('state, 'req) request_generator = 'state -> 'req option;;

let request_slave (ic, oc) req =
  IO.to_channel oc req true;
  let rep = IO.of_channel ic in
  rep
;;

let slave_loop (ic, oc) req_handler =
  let rec slave_loop () =
    try
      let req = IO.of_channel ic in
      IO.to_channel oc (req_handler req) true;
      slave_loop ()
    with End_of_file ->
      Log.say (Log.Log 0) "Connection closed by the master node." in
  slave_loop ()
;;

let master_loop req_gen req_handler s =
  let rec master_loop s =
    match req_gen s with
    | Some req ->
      let nstate = req_handler req in
      master_loop nstate
    | None -> s in
  master_loop s
;;
